@@ -1,18 +1,19 @@
 # testing/exclusions.py
-# Copyright (C) 2005-2017 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2019 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
 
+import contextlib
 import operator
-from ..util import decorator
+import re
+
 from . import config
 from .. import util
-import inspect
-import contextlib
-from sqlalchemy.util.compat import inspect_getargspec
+from ..util import decorator
+from ..util.compat import inspect_getargspec
 
 
 def skip_if(predicate, reason=None):
@@ -69,15 +70,15 @@ class compound(object):
 
     def matching_config_reasons(self, config):
         return [
-            predicate._as_string(config) for predicate
-            in self.skips.union(self.fails)
+            predicate._as_string(config)
+            for predicate in self.skips.union(self.fails)
             if predicate(config)
         ]
 
     def include_test(self, include_tags, exclude_tags):
         return bool(
-            not self.tags.intersection(exclude_tags) and
-            (not include_tags or self.tags.intersection(include_tags))
+            not self.tags.intersection(exclude_tags)
+            and (not include_tags or self.tags.intersection(include_tags))
         )
 
     def _extend(self, other):
@@ -86,13 +87,14 @@ class compound(object):
         self.tags.update(other.tags)
 
     def __call__(self, fn):
-        if hasattr(fn, '_sa_exclusion_extend'):
+        if hasattr(fn, "_sa_exclusion_extend"):
             fn._sa_exclusion_extend._extend(self)
             return fn
 
         @decorator
         def decorate(fn, *args, **kw):
             return self._do(config._current, fn, *args, **kw)
+
         decorated = decorate(fn)
         decorated._sa_exclusion_extend = self
         return decorated
@@ -112,10 +114,7 @@ class compound(object):
     def _do(self, cfg, fn, *args, **kw):
         for skip in self.skips:
             if skip(cfg):
-                msg = "'%s' : %s" % (
-                    fn.__name__,
-                    skip._as_string(cfg)
-                )
+                msg = "'%s' : %s" % (fn.__name__, skip._as_string(cfg))
                 config.skip_test(msg)
 
         try:
@@ -126,16 +125,20 @@ class compound(object):
             self._expect_success(cfg, name=fn.__name__)
             return return_value
 
-    def _expect_failure(self, config, ex, name='block'):
+    def _expect_failure(self, config, ex, name="block"):
         for fail in self.fails:
             if fail(config):
-                print(("%s failed as expected (%s): %s " % (
-                    name, fail._as_string(config), str(ex))))
+                print(
+                    (
+                        "%s failed as expected (%s): %s "
+                        % (name, fail._as_string(config), str(ex))
+                    )
+                )
                 break
         else:
             util.raise_from_cause(ex)
 
-    def _expect_success(self, config, name='block'):
+    def _expect_success(self, config, name="block"):
         if not self.fails:
             return
         for fail in self.fails:
@@ -143,13 +146,12 @@ class compound(object):
                 break
         else:
             raise AssertionError(
-                "Unexpected success for '%s' (%s)" %
-                (
+                "Unexpected success for '%s' (%s)"
+                % (
                     name,
                     " and ".join(
-                        fail._as_string(config)
-                        for fail in self.fails
-                    )
+                        fail._as_string(config) for fail in self.fails
+                    ),
                 )
             )
 
@@ -185,18 +187,26 @@ class Predicate(object):
             return predicate
         elif isinstance(predicate, (list, set)):
             return OrPredicate(
-                [cls.as_predicate(pred) for pred in predicate],
-                description)
+                [cls.as_predicate(pred) for pred in predicate], description
+            )
         elif isinstance(predicate, tuple):
             return SpecPredicate(*predicate)
         elif isinstance(predicate, util.string_types):
-            tokens = predicate.split(" ", 2)
-            op = spec = None
-            db = tokens.pop(0)
-            if tokens:
-                op = tokens.pop(0)
-            if tokens:
-                spec = tuple(int(d) for d in tokens.pop(0).split("."))
+            tokens = re.match(
+                r"([\+\w]+)\s*(?:(>=|==|!=|<=|<|>)\s*([\d\.]+))?", predicate
+            )
+            if not tokens:
+                raise ValueError(
+                    "Couldn't locate DB name in predicate: %r" % predicate
+                )
+            db = tokens.group(1)
+            op = tokens.group(2)
+            spec = (
+                tuple(int(d) for d in tokens.group(3).split("."))
+                if tokens.group(3)
+                else None
+            )
+
             return SpecPredicate(db, op, spec, description=description)
         elif util.callable(predicate):
             return LambdaPredicate(predicate, description)
@@ -209,11 +219,13 @@ class Predicate(object):
             bool_ = not negate
         return self.description % {
             "driver": config.db.url.get_driver_name()
-            if config else "<no driver>",
+            if config
+            else "<no driver>",
             "database": config.db.url.get_backend_name()
-            if config else "<no database>",
+            if config
+            else "<no database>",
             "doesnt_support": "doesn't support" if bool_ else "does support",
-            "does_support": "does support" if bool_ else "doesn't support"
+            "does_support": "does support" if bool_ else "doesn't support",
         }
 
     def _as_string(self, config=None, negate=False):
@@ -240,21 +252,21 @@ class SpecPredicate(Predicate):
         self.description = description
 
     _ops = {
-        '<': operator.lt,
-        '>': operator.gt,
-        '==': operator.eq,
-        '!=': operator.ne,
-        '<=': operator.le,
-        '>=': operator.ge,
-        'in': operator.contains,
-        'between': lambda val, pair: val >= pair[0] and val <= pair[1],
+        "<": operator.lt,
+        ">": operator.gt,
+        "==": operator.eq,
+        "!=": operator.ne,
+        "<=": operator.le,
+        ">=": operator.ge,
+        "in": operator.contains,
+        "between": lambda val, pair: val >= pair[0] and val <= pair[1],
     }
 
     def __call__(self, config):
         engine = config.db
 
         if "+" in self.db:
-            dialect, driver = self.db.split('+')
+            dialect, driver = self.db.split("+")
         else:
             dialect, driver = self.db, None
 
@@ -267,8 +279,9 @@ class SpecPredicate(Predicate):
             assert driver is None, "DBAPI version specs not supported yet"
 
             version = _server_version(engine)
-            oper = hasattr(self.op, '__call__') and self.op \
-                or self._ops[self.op]
+            oper = (
+                hasattr(self.op, "__call__") and self.op or self._ops[self.op]
+            )
             return oper(version, self.spec)
         else:
             return True
@@ -283,17 +296,9 @@ class SpecPredicate(Predicate):
                 return "%s" % self.db
         else:
             if negate:
-                return "not %s %s %s" % (
-                    self.db,
-                    self.op,
-                    self.spec
-                )
+                return "not %s %s %s" % (self.db, self.op, self.spec)
             else:
-                return "%s %s %s" % (
-                    self.db,
-                    self.op,
-                    self.spec
-                )
+                return "%s %s %s" % (self.db, self.op, self.spec)
 
 
 class LambdaPredicate(Predicate):
@@ -350,8 +355,9 @@ class OrPredicate(Predicate):
             conjunction = " and "
         else:
             conjunction = " or "
-        return conjunction.join(p._as_string(config, negate=negate)
-                                for p in self.predicates)
+        return conjunction.join(
+            p._as_string(config, negate=negate) for p in self.predicates
+        )
 
     def _negation_str(self, config):
         if self.description is not None:
@@ -381,18 +387,18 @@ def _server_version(engine):
 
     # force metadata to be retrieved
     conn = engine.connect()
-    version = getattr(engine.dialect, 'server_version_info', ())
+    version = getattr(engine.dialect, "server_version_info", None)
+    if version is None:
+        version = ()
     conn.close()
     return version
 
 
 def db_spec(*dbs):
-    return OrPredicate(
-        [Predicate.as_predicate(db) for db in dbs]
-    )
+    return OrPredicate([Predicate.as_predicate(db) for db in dbs])
 
 
-def open():
+def open():  # noqa
     return skip_if(BooleanPredicate(False, "mark as execute"))
 
 
@@ -410,24 +416,22 @@ def future(fn, *arg):
 
 
 def fails_on(db, reason=None):
-    return fails_if(Predicate.as_predicate(db), reason)
+    return fails_if(db, reason)
 
 
 def fails_on_everything_except(*dbs):
-    return succeeds_if(
-        OrPredicate([
-            Predicate.as_predicate(db) for db in dbs
-        ])
-    )
+    return succeeds_if(OrPredicate([Predicate.as_predicate(db) for db in dbs]))
 
 
 def skip(db, reason=None):
-    return skip_if(Predicate.as_predicate(db), reason)
+    return skip_if(db, reason)
 
 
 def only_on(dbs, reason=None):
     return only_if(
-        OrPredicate([Predicate.as_predicate(db) for db in util.to_list(dbs)])
+        OrPredicate(
+            [Predicate.as_predicate(db, reason) for db in util.to_list(dbs)]
+        )
     )
 
 
@@ -437,7 +441,6 @@ def exclude(db, op, spec, reason=None):
 
 def against(config, *queries):
     assert queries, "no queries sent!"
-    return OrPredicate([
-        Predicate.as_predicate(query)
-        for query in queries
-    ])(config)
+    return OrPredicate([Predicate.as_predicate(query) for query in queries])(
+        config
+    )
